from sklearn.base import TransformerMixin, BaseEstimator
from sklearn.preprocessing import LabelBinarizer, StandardScaler
import pandas as pd
import datetime

class Interactions(BaseEstimator, TransformerMixin):

    def __init__(self, these=(), with_these=()):
        """
        We're assuming that `these` are categorical, and `with_these` are numerical.

        """
        self.these = these
        self.with_these = with_these

    def fit(self, X, y=None):
        self.binarizers = {field: LabelBinarizer().fit(X[field]) for field in self.these}
        return self

    def transform(self, X):
        df = X.copy()
        index = df.index
        dfs = []
        for field, binarizer in self.binarizers.items():
            classes = binarizer.classes_
            column_classes = [classes[-1]] if len(classes) <= 2 else classes
            onehot = pd.DataFrame(data=binarizer.transform(df[field]),
                                          index=index,
                                          columns=[field + '_' + '_'.join(str(col).lower().split())
                                          for col in column_classes])
            for with_field in self.with_these:
                multiplied = onehot.multiply(
                        df[with_field],
                        axis='index')
                multiplied.columns = [col + '_with_' + with_field for col in onehot.columns]
                dfs.append(multiplied)
        return pd.concat(dfs, axis=1)

