import numpy as np
from sklearn.base import TransformerMixin, BaseEstimator
from sklearn.feature_extraction.text import CountVectorizer
from pandas import DataFrame, concat

class MCountVectorizer(BaseEstimator, TransformerMixin):
   
    def fit(self, X, y=None):
        self.vectorizers = [(col, CountVectorizer().fit(X[col])) for col in X.columns]
        self.columns = [col + '=' + field for col, vec in self.vectorizers for field in sorted(vec.vocabulary_, key=lambda x: vec.vocabulary_[x])]
        return self

    def transform(self, X, y=None):
        data = []
        for col, vectorizer in self.vectorizers:
            data.append(
                vectorizer.transform(X[col]).toarray()
            )
        return DataFrame(np.hstack(data), columns=self.columns, index=X.index)
            
