from sklearn.base import TransformerMixin, BaseEstimator
from sklearn.preprocessing import LabelBinarizer
from pandas import DataFrame, concat


class MultiLabelEncoder(BaseEstimator, TransformerMixin):

    def __init__(self, fields=(), weights={}):
        self.fields = fields
        self.weights = weights

    def fit(self, df, y=None):
        if not list(self.fields):
            self.fields = list(df.columns.values)
        self.binarizers = [(field, LabelBinarizer().fit(df[field])) for field in self.fields]
        return self

    def transform(self, df, y=None):
        new_df = df.drop(self.fields, axis=1)
        dfs = []
        for field, binarizer in self.binarizers:
            classes = [binarizer.classes_[-1]] if len(binarizer.classes_) <= 2 else binarizer.classes_
            dfs.append(
                DataFrame(data=binarizer.transform(df[field]),
                          index=new_df.index,
                          columns=[field + '=' + '_'.join(str(col).lower().split()) for col in classes])
                * self.weights.get(field, 1)
            )
        return concat([new_df] + dfs, axis=1)
