from sklearn.base import TransformerMixin, BaseEstimator
import pandas as pd
from operator import attrgetter,  add
import datetime

class Operator(BaseEstimator, TransformerMixin):

    def __init__(self, left=None, right=None, accessor_right=None,
                 accessor_left=None, drop=None, feature_name=None,
                 operator=add):
        self.right = right
        self.left = left
        self.drop = drop
        self.accessor_right=accessor_right
        self.accessor_left=accessor_left
        self.feature_name=feature_name
        self.operator = operator

    def fit(self, df, y=None):
        if self.feature_name is None:
            self.feature_name = str(self.left) + '_' + self.operator.__name__ + '_' + str(self.right)
        return self
    
    def _determine_field(self, X, field, accessor):
        if field in X:
            if accessor:
                return attrgetter(accessor)(X[field])
            else:
                return X[field]
        else:
            #assume field is a number or something that can be subtracted
            return field

    def transform(self, df):
        if not self.left or not self.right:
            return df
        X = df.copy()
        left_ = self._determine_field(df, self.left, self.accessor_left)
        right_ = self._determine_field(df, self.right, self.accessor_right)
        X[self.feature_name] = self.operator(left_, right_)
        return X.drop(self.drop, axis=1) if self.drop else X
