from sklearn.base import TransformerMixin, BaseEstimator
from sklearn.preprocessing import StandardScaler
from pandas import DataFrame, concat

class Pandify(BaseEstimator, TransformerMixin):
    """
    Apply a sklearn estimator to each column of a DataFrame and return the
    result as a DataFrame with the same index and columns.

    """
    def __init__(self, estimator=None):
        self.estimator = estimator
        self.fitted = False

    def fit(self, X, y = None):
        if self.estimator is None:
            raise ValueError('estimator can not be None')
        if not self.fitted:
            self.estimator = self.estimator()
        self.estimator.fit(X)
        self.fitted = True
        return self

    def transform(self, X, y = None):
        return DataFrame(self.estimator.transform(X),
                         index=X.index,
                         columns=X.columns)
