from sklearn.base import TransformerMixin, BaseEstimator
from pandas import DataFrame

class TopNValues(BaseEstimator, TransformerMixin):

    def __init__(self, cols=[], n=10, other_value="other"):
        self.cols = cols
        self.n = n
        self.other_value = other_value

    def fit(self, df, y=None):
        self.mapping = {}
        for c in self.cols:
            value_counts = df[c].value_counts()
            top_n_vals = value_counts.iloc[:self.n].index
            self.mapping[c] = dict([(x, x) for x in top_n_vals])
        return self

    def transform(self, df, y=None):
        df = df.copy()
        for c in self.cols:
            df[c] = df[c].map(lambda x: self.mapping[c].get(x,self.other_value))
        return df